import * as React from 'react';
import * as PropTypes from 'prop-types';

import classNames from '../../../utils/classnames';
import FormCell from '../formcell';

export default function Input({
  // 系统属性
  id,
  className,
  style,
  events,
  // 组件属性
  label,
  labelVisible,
  name,
  defaultValue,
  password,
  placeholder,
  type,
  maxLength,
  disabled,
  focus: autoFocus,
  layout,
  requiredFlag,
}) {
  const cls = classNames({
    'weda-ui': true,
    'weda-input': true,
    [className]: className,
  });
  const subCls = classNames({
    'weui-cell': true,
    'weui-cell_active': true,
    'weui-cell_form': true,
    'weui-cell_disabled': disabled,
  });

  const [value, setValue] = React.useState(defaultValue);
  // 两次默认值不同时, 需要刷新value
  const prevDefaultValueRef = React.useRef();
  React.useEffect(() => {
    if (prevDefaultValueRef.current !== defaultValue) {
      prevDefaultValueRef.current = defaultValue;
      setValue(defaultValue);
    }
  });

  const onChange = function (e) {
    if (maxLength > 0 && e.target.value.length > maxLength) {
      return;
    }
    const newValue = e.target.value;
    setValue(newValue);
    events.change({ value: newValue }, { originEvent: e });
  };

  const onClear = function (e) {
    events.clear({ originValue: value }, { originEvent: e });
    setValue('');
    e.preventDefault();
  };

  const onConfirm = function (e) {
    if (e.key === 'enter' || e.keyCode === 13) {
      events.confirm({ value: e.target.value }, { originEvent: e });
    }
  };

  const onFocus = function (e) {
    events.focus({ value: e.target.value }, { originEvent: e });
  };

  const onBlur = function (e) {
    events.blur({ value: e.target.value }, { originEvent: e });
  };

  return (
    <FormCell
      className={cls}
      style={style}
      label={labelVisible ? label : null}
      layout={layout}
      multiCell={false}
      requiredFlag={requiredFlag}
    >
      <div className={subCls}>
        <div className="weui-cell__bd weui-flex">
          <input
            className="weui-input"
            type={getType(type, password)}
            placeholder={placeholder}
            value={value}
            disabled={disabled}
            name={name}
            autoFocus={autoFocus}
            onChange={onChange}
            onFocus={onFocus}
            onBlur={onBlur}
            onKeyDown={onConfirm}
          />
          <button
            className="weui-btn_reset weui-btn_icon weui-btn_input-clear"
            onMouseDown={onClear}
          >
            <i className="weui-icon-clear"></i>
          </button>
        </div>
      </div>
    </FormCell>
  );
}

Input.propTypes = {
  // 系统属性
  id: PropTypes.string,
  className: PropTypes.string,
  style: PropTypes.string,
  events: PropTypes.object,
  // 组件属性
  label: PropTypes.string,
  labelVisible: PropTypes.bool,
  name: PropTypes.string,
  defaultValue: PropTypes.string,
  password: PropTypes.bool,
  placeholder: PropTypes.string,
  type: PropTypes.oneOf(['text', 'number', 'idcard', 'digit']),
  maxLength: PropTypes.number,
  disabled: PropTypes.bool,
  focus: PropTypes.bool,
  layout: PropTypes.oneOf(['horizontal', 'vertical']),
  requiredFlag: PropTypes.bool,
};

Input.defaultProps = {
  // 系统属性
  events: {},
  // 组件属性
  label: '标题',
  labelVisible: true,
  name: 'formInput',
  defaultValue: '',
  password: false,
  placeholder: '请输入',
  type: 'text',
  maxLength: 140,
  disabled: false,
  focus: false,
  layout: 'horizontal',
  requiredFlag: false,
};

function getType(type, password) {
  if (password) {
    return 'password';
  }
  switch (type) {
    case 'idcard':
      return 'text';
    case 'digit':
      return 'number';
    default:
      return type;
  }
}
