import * as React from 'react';
import * as PropTypes from 'prop-types';
import weui from '../../../utils/weui';

import classNames from '../../../utils/classnames';
import isObjectEqual from '../../../utils/isObjectEqual';
import FormCell from '../formcell';
import { timePicker } from './time';
import { regionPicker } from './region';

export default function Select({
  // 系统属性
  id,
  className,
  style,
  events,
  // 组件属性
  label,
  labelVisible,
  name,
  placeholder,
  layout,
  mode,
  disabled,
  requiredFlag,
  // selector mode
  range,
  defaultValue,
  // date mode
  startDate,
  endDate,
  defaultDate,
  // time mode
  startTime,
  endTime,
  defaultTime,
  // region mode
  defaultRegion,
  separator,
}) {
  const [date, setDate] = React.useState(
    decodeIsoDatetime(defaultDate, 'date')
  );
  const [time, setTime] = React.useState(
    decodeIsoDatetime(defaultTime, 'time')
  );
  const [region, setRegion] = React.useState(
    defaultRegion?.join(separator) || ''
  );
  const [selectedValue, setSelectedValue] = React.useState(defaultValue);

  // 两次默认值不同, 需要刷新
  const prevDefaultRef = React.useRef();
  React.useEffect(() => {
    console.log('select', 'effect');
    let hasChange = false;
    if (prevDefaultRef.current?.defaultValue !== defaultValue) {
      setSelectedValue(defaultValue);
      hasChange = true;
    }
    if (prevDefaultRef.current?.defaultDate !== defaultDate) {
      setDate(decodeIsoDatetime(defaultDate, 'date'));
      hasChange = true;
    }
    if (prevDefaultRef.current?.defaultTime !== defaultTime) {
      setTime(decodeIsoDatetime(defaultTime, 'time'));
      hasChange = true;
    }
    if (!isObjectEqual(prevDefaultRef.current?.defaultRegion, defaultRegion)) {
      setRegion(defaultRegion?.join(separator) || '');
      hasChange = true;
    }
    if (hasChange) {
      prevDefaultRef.current = {
        defaultDate,
        defaultTime,
        defaultRegion,
      };
    }
  });

  const isFlex = layout !== 'vertical';
  const cls = classNames({
    'weda-ui': true,
    'weda-select': true,
    [className]: className,
  });
  const subCls = classNames({
    'weui-cell': true,
    'weui-cell_active': true,
    'weui-cell_form': true,
    'weui-flex__item': isFlex,
    'weui-cell_disabled': disabled,
  });

  const onDateclick = function () {
    const options = {};
    !!date && (options.defaultDate = date);
    !!startDate && (options.start = decodeIsoDatetime(startDate, 'date'));
    !!endDate && (options.end = decodeIsoDatetime(endDate, 'date'));

    weui.datePicker({
      ...options,
      onConfirm(result) {
        const value = `${result[0]}-${result[1]}-${result[2]}`;
        setDate(value);
        events.change({ value });
      },
    });
  };

  const onTimeClick = function () {
    const options = {};
    !!time && (options.defaultTime = time);
    !!startTime && (options.start = decodeIsoDatetime(startTime, 'time'));
    !!endTime && (options.end = decodeIsoDatetime(endTime, 'time'));

    timePicker({
      ...options,
      onConfirm(result) {
        const value = `${result[0]}:${result[1]}`;
        setTime(value);
        events.change({ value });
      },
    });
  };

  const onSelect = function () {
    weui.picker(range, {
      defaultValue: [selectedValue || range[0].value],
      onConfirm(result) {
        const item = result[0];
        setSelectedValue(item?.value);
        events.change({ value: item?.value });
      },
    });
  };

  const onRegionClick = function () {
    regionPicker({
      defaultValue:
        defaultRegion.length <= 0
          ? ['北京市', '北京市', '东城区']
          : defaultRegion,
      onConfirm(result) {
        const value = result.map((item) => item.value);
        setRegion(value.join(separator));
        events.change({ value });
      },
    });
  };

  const onClick = function () {
    if (disabled) {
      return;
    }

    switch (mode) {
      case 'date': {
        onDateclick();
        break;
      }
      case 'time': {
        onTimeClick();
        break;
      }
      case 'selector': {
        onSelect();
        break;
      }
      case 'region': {
        onRegionClick();
        break;
      }
      default: {
        break;
      }
    }
  };

  const result = {
    selector: range.find((item) => item.value === selectedValue)?.label,
    date: date,
    time: time,
    region: region,
  };

  return (
    <FormCell
      className={cls}
      style={style}
      label={labelVisible ? label : null}
      layout={layout}
      multiCell={false}
      requiredFlag={requiredFlag}
    >
      <div className={subCls}>
        <div className="weui-cell__bd weui-flex">
          <input
            onClick={onClick}
            className="weui-input"
            type="select"
            name={name}
            placeholder={placeholder}
            value={result[mode]}
            disabled={disabled}
            autoComplete={'off'}
            readOnly={true}
          />
        </div>
      </div>
    </FormCell>
  );
}

Select.propTypes = {
  // 系统属性
  id: PropTypes.string,
  className: PropTypes.string,
  style: PropTypes.string,
  events: PropTypes.objectOf(PropTypes.func),
  // 组件属性
  label: PropTypes.string,
  labelVisible: PropTypes.boolean,
  name: PropTypes.string,
  placeholder: PropTypes.string,
  layout: PropTypes.string,
  mode: PropTypes.string,
  disabled: PropTypes.boolean,
  // selector mode
  range: PropTypes.array,
  defaultValue: PropTypes.string,
  // date mode
  startDate: PropTypes.string,
  endDate: PropTypes.string,
  defaultDate: PropTypes.string,
  // time mode
  startTime: PropTypes.string,
  endTime: PropTypes.string,
  defaultTime: PropTypes.string,
  // region mode
  defaultRegion: PropTypes.array,
  separator: PropTypes.string,
  requiredFlag: PropTypes.bool,
};

Select.defaultProps = {
  // 系统属性
  events: {},
  // 组件属性
  label: '标题',
  labelVisible: true,
  name: 'formSelect',
  placeholder: '请选择',
  layout: 'horizontal',
  mode: 'selector',
  disabled: false,
  requiredFlag: false,
  // selector mode
  range: [
    { name: '选项一', value: 'first' },
    { name: '选项二', value: 'second' },
  ],
  // region mode
  separator: ',',
};

function decodeIsoDatetime(isoString, type) {
  if (!isoString) {
    return null;
  }
  const datetime = new Date(
    isNaN(parseInt(isoString)) ? isoString : parseInt(isoString)
  );
  if (type === 'time') {
    return `${datetime.getHours()}:${datetime.getMinutes()}`;
  } else {
    return `${datetime.getFullYear()}-${
      datetime.getMonth() + 1
    }-${datetime.getDate()}`;
  }
}
