import * as React from 'react';
import {
  Upload,
  Button,
  Icon,
  message,
  ErrorTip,
  ImagePreview,
} from 'tea-component';
import classNames from '../../../utils/classnames';
import { getCloudInstance, getTempFileURL } from '../../../utils/tcb';

// 云存储路径
const CLOUD_PATH = 'business';
// 组件类前缀
const CLASS_PREFIX = 'g-uploader';
// 默认图片类型
const IMAGE_TYPES = [
  'image/jpg',
  'image/png',
  'image/tif',
  'image/bmp',
  'image/jepg',
  'image/tiff',
];

/**
 * 上传图片-官方组件
 */
export function UploaderPc(props) {
  const {
    id = `uploader-${new Date().valueOf()}`,
    className,
    title = '',
    tips = '',
    btnTitle = '上传图片',
    maxUploadCount = 9,
    maxSize = 10,
    initialValue = [],
    acceptTypes = IMAGE_TYPES,
    events = {},
    readOnly = false,
    onChange,
    ...rest
  } = props;
  const cls = classNames({
    'weda-ui': true,
    [CLASS_PREFIX]: true,
    [className]: className,
  });

  // 上传中
  const [uploading, setUploading] = React.useState(false);
  //上传进度
  const [progress, setProgress] = React.useState(0);
  // 文件列表
  const [fileIDList, setfileIDList] = React.useState(
    Array.isArray(initialValue) ? initialValue.filter((d) => d) : []
  );

  console.log('====fileIDList', fileIDList);

  // 值变化事件
  React.useEffect(() => {
    events.change && events.change({ value: fileIDList });
    onChange && onChange(fileIDList);
  }, [fileIDList]);

  // 方法：上传前，判断图片大小、数量是否满足，取消默认组件的上传事件，用自定义的 tcb 上传方法
  const beforeHandle = (file, fileList, isAccepted, error) => {
    if (!isAccepted) {
      try {
        const errorList = [];
        if (error.find((item) => item?.code === 'file-invalid-type')) {
          errorList.push('上传图片类型错误');
        }
        if (error.find((item) => item?.code === 'file-too-large')) {
          errorList.push(`上传图片大小不能超过${maxSize}M`);
        }
        message.error({ content: errorList.join(', ') });
      } catch (e) {}
      return false;
    }
    uploadTcb(file);
    return false;
  };

  // 自定义上传到 Tcb的Cos
  const uploadTcb = (file) => {
    setUploading(true);
    getCloudInstance().then(async (tcb) => {
      try {
        const fileType = file.type.split('/')?.[1];
        const cloudPath = `${CLOUD_PATH}/uploader-${Date.now()}.${fileType}`;
        const { fileID } = await tcb.uploadFile({
          cloudPath,
          filePath: file,
          onUploadProgress: onProgress,
        });
        fileID && setfileIDList((list) => [...list, fileID]);
        fileID && events.success && events.success({ value: fileID, file });
      } catch (e) {
        events.error && events.error({ value: e, file });
      } finally {
        setUploading(false);
        setProgress(0);
      }
    });
  };

  // 计算上传进度
  const onProgress = (progressEvent) => {
    let percentCompleted = 0;
    try {
      percentCompleted = Math.round(
        (progressEvent.loaded * 100) / progressEvent.total
      );
      percentCompleted < 100 ? percentCompleted : 100;
    } catch (e) {}
    setProgress(percentCompleted);
  };

  // 删除图片
  const deleteHandle = (fileID) => {
    setfileIDList((list) => list.filter((d) => d !== fileID));
  };

  // 图片上限数量提示
  const maxUploadHandle = () => {
    message.warning({ content: '已达到上传图片数量上限' });
  };

  // 转换了的属性
  const extraProps = {};
  extraProps['accept'] =
    acceptTypes.includes('image/*') || acceptTypes.length === 0
      ? IMAGE_TYPES
      : Array.from(new Set(acceptTypes));
  tips && (extraProps['title'] = tips);
  maxSize && (extraProps['maxSize'] = maxSize * 1024 * 1024);

  return (
    <div className={cls} id={id} {...rest}>
      {!readOnly && (
        <div className={`${CLASS_PREFIX}__header`}>
          <div className={`${CLASS_PREFIX}__title`}>{title}</div>
          <div className={`${CLASS_PREFIX}__description`}>{tips || ''}</div>
        </div>
      )}
      <div className={`${CLASS_PREFIX}__body`}>
        {!readOnly && (
          <div className={`${CLASS_PREFIX}__input-box`}>
            {fileIDList.length < maxUploadCount ? (
              <Upload {...extraProps} beforeUpload={beforeHandle}>
                <Button role="upload">{btnTitle}</Button>
              </Upload>
            ) : (
              <Button role="upload" onClick={maxUploadHandle}>
                {btnTitle}
              </Button>
            )}
          </div>
        )}

        <div className={`${CLASS_PREFIX}__image-list`}>
          {fileIDList.map((d) => (
            <div className={`${CLASS_PREFIX}__image-box`} key={d}>
              <TcbImage fileID={d} />
              {!readOnly && (
                <div className={`${CLASS_PREFIX}__action`}>
                  <Button
                    onClick={() => deleteHandle(d)}
                    className={`${CLASS_PREFIX}__btn`}
                  >
                    删除
                  </Button>
                </div>
              )}
            </div>
          ))}
          {uploading && (
            <div className={`${CLASS_PREFIX}__image-box`} key="_place_image">
              <Icon type="loading" />
              <span>上传{progress}%...</span>
            </div>
          )}
        </div>
      </div>
    </div>
  );
}

/**
 * 单张图片回显组件
 */
const TcbImage = ({ fileID }) => {
  const [src, setSrc] = React.useState('');
  const [isError, setIsError] = React.useState(false);
  React.useEffect(() => {
    const getSrc = async () => {
      try {
        const _src = await getTempFileURL(fileID);
        _src ? setSrc(_src) : setIsError(true);
      } catch (e) {
        message.error('上传图片失败');
      }
    };
    getSrc();
  }, [fileID]);
  return isError ? (
    <ErrorTip />
  ) : src ? (
    <ImagePreview
      previewSrc={src}
      src={src}
      className={`${CLASS_PREFIX}__image`}
    />
  ) : null;
};
