import React, { useEffect, useState } from 'react';
import { Upload, Progress, Modal, Input, Button } from 'antd';
import { InboxOutlined } from '@ant-design/icons';
import BraftEditor from 'braft-editor';
import MaxLength from 'braft-extensions/dist/max-length';
import { useSetState } from 'react-use';
import { func, object, string, bool, oneOf, number, array } from 'prop-types';
import { ContentUtils } from 'braft-utils';
import classNames from '../../utils/classnames';
import FormCell from '../form/formcell';
import { getCloudInstance, getTempFileURL } from '../../utils/tcb';
import { message } from 'tea-component';

import './index.css';
import 'antd/dist/antd.css';
import 'braft-editor/dist/index.css';

// 引入拖拽
const { Dragger } = Upload;

// 云存储路径
const CLOUD_PATH = 'business';

// 默认图片类型
const IMAGE_TYPES = [
  'image/jpg',
  'image/png',
  'image/tif',
  'image/bmp',
  'image/jepg',
  'image/tiff',
];

export default function RichText({
  // 系统属性
  id,
  style,
  className,
  events,
  // 组件属性
  label,
  labelVisible,
  initialValue,
  readOnly,
  layout,
  requiredFlag,
  maxLength,
  onChange,
  maxSize,
  acceptTypes,
}) {
  const [editorState, setEditorState] = useState();

  const cls = classNames({
    'weda-ui': true,
    'weda-richText': true,
    [className]: className,
  });
  const subCls = classNames({
    'weui-cell': true,
    'weui-cell_active': true,
    'weui-cell_form': true,
    'weui-cell_disabled': true,
  });

  const maxLengthOptions = {
    defaultValue: 32768, // 指定默认限制数，如不指定则为Infinity(无限)
  };
  BraftEditor.use(MaxLength(maxLengthOptions));

  const onDataChange = function (value) {
    events.change && events.change({ value });
    onChange && onChange(value);
  };

  useEffect(() => {
    const state = BraftEditor.createEditorState(initialValue);
    setEditorState(state);
  }, [initialValue]);

  const extendControls = [
    {
      key: 'antd-uploader',
      type: 'component',
      component: (
        <CustomUploader
          acceptTypes={acceptTypes}
          maxSize={maxSize}
          onChange={(url) => {
            const state = ContentUtils.insertMedias(editorState, [
              { type: 'IMAGE', url },
            ]);
            setEditorState(state);
          }}
        />
      ),
    },
  ];

  // 修改img显示逻辑
  const blockRenderFn = (contentBlock, { editor, editorState }) => {
    if (contentBlock.getType() === 'atomic') {
      const entity = editorState
        .getCurrentContent()
        .getEntity(contentBlock.getEntityAt(0));
      if (entity.getType() === 'IMAGE') {
        return {
          component: RichTextImg, // 替换img的组件
          editable: false, // editable并不代表组件内容实际可编辑，建议设置为false
          props: { editor, editorState }, // 传入的内容可以在组件中通过this.props.blockProps获取到
        };
      }
    }
  };

  return (
    <FormCell
      className={cls}
      label={labelVisible ? label : null}
      layout={layout}
      multiCell={false}
      requiredFlag={requiredFlag}
    >
      <div className={subCls}>
        <div style={{ border: '1px solid #d1d1d1' }} role="richtext">
          <BraftEditor
            key={id}
            style={style}
            value={editorState}
            readOnly={readOnly}
            blockRendererFn={blockRenderFn}
            onChange={(s) => {
              if (editorState.toText().length <= maxLength) {
                setEditorState(s);
                onDataChange(s.toHTML());
              }
            }}
            excludeControls={['media', 'fullscreen']}
            extendControls={extendControls}
            maxLength={maxLength}
            onReachMaxLength={() =>
              message.warning({ content: `字符长度超过上限${maxLength}` })
            }
          />
        </div>
      </div>
    </FormCell>
  );
}

export const CustomUploader = (props) => {
  const { onChange = () => {}, acceptTypes, maxSize } = props;
  const [
    { fileList, percent, visible, uploading, inputUrl },
    setState,
  ] = useSetState({
    fileList: [],
    percent: 0,
    visible: false,
    uploading: false,
    inputUrl: '',
  });

  // 计算上传进度
  const onProgress = (progressEvent) => {
    let percentCompleted = 0;
    try {
      percentCompleted = Math.round(
        (progressEvent.loaded * 100) / progressEvent.total
      );
      percentCompleted < 100 ? percentCompleted : 100;
    } catch (e) {}
    setState({ uploading: true, percent: percentCompleted });
  };

  return (
    <>
      <button
        type="button"
        data-title="媒体上传"
        className="control-item button"
        onClick={() => setState({ visible: true })}
      >
        <svg
          viewBox="64 64 896 896"
          focusable="false"
          data-icon="file-image"
          width="1em"
          height="1em"
          fill="currentColor"
          aria-hidden="true"
        >
          <path
            d="M534 352V136H232v752h560V394H576a42 42 0 01-42-42zm-134 50c22.1 0 40 17.9 40 40s-17.9 40-40 40-40-17.9-40-40 17.9-40 40-40zm296 294H328.1c-6.7 0-10.4-7.7-6.3-12.9l99.8-127.2a8 8 0 0112.6 0l41.1 52.4 77.8-99.2a8.1 8.1 0 0112.7 0l136.5 174c4.1 5.2.4 12.9-6.3 12.9z"
            fill="#e6f7ff"
          ></path>
          <path
            d="M854.6 288.6L639.4 73.4c-6-6-14.1-9.4-22.6-9.4H192c-17.7 0-32 14.3-32 32v832c0 17.7 14.3 32 32 32h640c17.7 0 32-14.3 32-32V311.3c0-8.5-3.4-16.7-9.4-22.7zM602 137.8L790.2 326H602V137.8zM792 888H232V136h302v216a42 42 0 0042 42h216v494z"
            fill="#006eff"
          ></path>
          <path
            d="M553.1 509.1l-77.8 99.2-41.1-52.4a8 8 0 00-12.6 0l-99.8 127.2a7.98 7.98 0 006.3 12.9H696c6.7 0 10.4-7.7 6.3-12.9l-136.5-174a8.1 8.1 0 00-12.7 0zM360 442a40 40 0 1080 0 40 40 0 10-80 0z"
            fill="#006eff"
          ></path>
        </svg>
      </button>
      <Modal
        centered
        closable={true}
        visible={visible}
        footer={null}
        onCancel={() => setState({ visible: false })}
      >
        <p>输入链接插入图片</p>
        <div className="richText-upload-link">
          <Input
            className="flex-auto"
            value={inputUrl}
            placeholder="输入图片地址"
            onChange={(e) => setState({ inputUrl: e.target.value })}
          />
          &nbsp;
          <Button
            type="primary"
            onClick={() => {
              onChange(inputUrl);
              setState({ visible: false });
            }}
          >
            确认
          </Button>
        </div>
        <p style={{ marginBottom: '16px' }}>或拖拽上传插入图片</p>
        <Dragger
          fileList={fileList}
          listType="picture"
          accept={acceptTypes.join()}
          beforeUpload={(file) => {
            const typeRegex = new RegExp(`^((${acceptTypes.join(')|(')}))$`);
            if (!typeRegex.test(file.type)) {
              message.error({
                content: '文件格式不支持',
              });
              return false;
            }
            if (file.size > maxSize * 1024 * 1024) {
              message.error({
                content: `文件大小超过上限${maxSize}M`,
              });
              return false;
            }
            setState({ uploading: true, percent: 0 });
            getCloudInstance().then(async (tcb) => {
              try {
                const fileType = file.type.split('/')?.[1];
                const cloudPath = `${CLOUD_PATH}/uploader-${Date.now()}.${fileType}`;
                const { fileID } = await tcb.uploadFile({
                  cloudPath,
                  filePath: file,
                  onUploadProgress: onProgress,
                });
                onChange(fileID);
                setState({
                  visible: false,
                  fileList: [
                    {
                      uid: fileID,
                      name: file.name,
                      status: 'done',
                    },
                  ],
                });
              } catch (e) {
                message.error({
                  content: `上传图片失败：${e.message || '未知原因'}`,
                });
              } finally {
                setState({ uploading: false, percent: 100 });
              }
            });
            return false;
          }}
        >
          <p className="ant-upload-drag-icon">
            <InboxOutlined />
          </p>
          <p className="ant-upload-text">点击或拖拽图片上传</p>
        </Dragger>
        {uploading && (
          <Progress style={{ paddingTop: '10px' }} percent={percent} />
        )}
      </Modal>
    </>
  );
};

export const RichTextImg = ({ contentState, block }) => {
  const blockData = contentState.getEntity(block.getEntityAt(0)).getData();
  const { url } = blockData || {};
  const [src, setSrc] = useState('');

  useEffect(() => {
    const getSrc = async () => {
      try {
        const _src = await getTempFileURL(url);
        setSrc(_src || '');
      } catch (e) {
        message.error({
          content: `显示图片失败：${e.message || '未知原因'}`,
        });
      }
    };
    getSrc();
  }, [url]);
  return <img src={src} />;
};

RichText.propTypes = {
  // 系统属性
  id: string,
  style: string,
  value: string,
  className: string,
  events: object,
  // 组件属性
  label: string,
  labelVisible: bool,
  name: string,
  initialValue: string,
  readOnly: bool,
  layout: oneOf(['horizontal', 'vertical']),
  requiredFlag: bool,
  maxLength: number,
  onChange: func,
  maxSize: number,
  acceptTypes: array,
};

RichText.defaultProps = {
  // 系统属性
  events: {},
  // 组件属性
  label: '标题',
  labelVisible: true,
  name: 'formRichText',
  initialValue: '',
  readOnly: false,
  layout: 'vertical',
  requiredFlag: false,
  maxSize: 10,
  acceptTypes: IMAGE_TYPES,
};

CustomUploader.propTypes = {
  onChange: func,
  maxSize: number,
  acceptTypes: array,
};

CustomUploader.defaultProps = {
  maxSize: 10,
  acceptTypes: IMAGE_TYPES,
};

RichTextImg.propTypes = {
  contentState: object,
  block: object,
};

RichTextImg.defaultProps = {};
